
# A-AUTOSAR COMMUNICATION WRAPPERS (SUB_PUB_UTILS)

The purpose of this **header-only** library is to facilitate running applications that need to receive and publish data on A-AUTOSAR platforms, by hiding publishing and subscribing logic

# Installation
**SUB_PUB_UTILS** lib is header-only, so clone the repo and and add the include directory to CMAKE include path
for example if lib is cloned in `/home/utils` add `include_directories(/home/utils/include/)` to `CMakeLists.txt`, U can also use relative path, relative to your current project


# Project Structure
**SUB_PUB_UTILS** lib is header only library with only 6 files.
This is how it is structured:

    root
        |--include
            |--sub
                |--SubDataWrapper.h
                |--SubWrapper.h
                |--SubWrapperHelper.h
            |--pub
                |--PubDataWrapper.h
                |--PubWrapper.h
                |--PubWrapperHelper.h

## How to use Sub(Subscription)

If your application is in need of receiving data from other sources, **Sub**  is what you want to use. In order to use **Sub** you need to identify the following:
 - Proxy type that delivers the data you need (_proxy)
 - proxy identifier string (_id)
 - Event type within _proxy that you need to subscribe to (_event)
 - Data type that will be passed to you after subscribing to event(_dataType)
 - Service instance ID (_serviceInstanceID)
 - max sample count (_maxSampleCount)
 - logger (_logger)

After identifying above follow these 3 steps

 1. You need to create a derived class that inherits from **SubDataWrapper** and Implement **Process** method, **Process** method is the method that gets called each time the subscribed _event's callback is called, you can use the data inside **Process** however you like.

 2.  You need to create a derived class that inherits from **SubWrapper** and implement **GetEvent** method, also make sure you provide **SubWrapper** constructor with proper data and **SubDataWrapper**  derived class from 1.

 3. You need to call **Init** method on the class you just created in 2 in your application.

**Here is an example:**
in this example 

> _proxy = VisualizationProxy 
> _id =  visualization/visualization/CameraH265ImageRPort
> _event = CameraH265ImagePDataViz
> _dataType = CameraH265Image
> _serviceInstanceID = "1"  _maxSampleCount = 10
> _logger = logger

Step 1

    #include "sub/SubDataWrapper.h"
    template <typename DataType>
    class SampleSubDataWrapper :  public subDataWrapper<DataType>
    {
    	void Process(ara::com::SamplePtr<DataType const>* ptr) override
        {
    	    auto sample = ptr->Get();
        	std::cout << "[Wrapper test]############# " << sample->header.frameId << std::endl;
        }
    };    

----
Step 2
    
    #include "sub/SubWrapper.h"
    using Proxy = huawei::ap::sample::proxy::VisualizationProxy; 
    using Event = ara::com::internal::proxy::event::EventAdapter<::CameraH265Image>;	
    class sampleWrapper : public SubWrapper<Proxy, Event, CameraH265Image>	
    {	
        public:	
       	SampleSubDataWrapper<CameraH265Image> s;	
      	sampleWrapper(const std::string& name, ara::log::Logger& log, std::string serviceInstanceId  = "1", uint32_t maxSampleCount = 10) :	
        		SubWrapper(s, name, log, serviceInstanceId, maxSampleCount) {}	
        private:	
        	Event& GetEvent() override	
        	{	return _proxy->CameraH265ImagePDataViz;	}
    };
---
Step 3:
Then In the application we can do something like this

    int main()
    {
    	SampleWrapper s("visualization/visualization/CameraH265ImageRPort", _logger);
    	s.Init();
    
    	while(1)
    	{
    		std::this_thread::sleep_for(std::chrono::milliseconds(10));
    	}
    }

  After this when ever data is received, the **Process** function of **SampleSubDataWrapper** is called.

After we are done with Proxy we can call **Destroy** to unsubscribe from the events;


## Sub Helper

Instead of doing step 2 there is a macro in code **GENERATE_SUB_WRAPPER** that can generate the  code for you given the proper parameters.

It is recommended that you use this macro for speed of development

Step 2:

     GENERATE_SUB_WRAPPER(SampleWrapper, SampleSubDataWrapper, VisualizationProxy, CameraH265Image, CameraH265ImagePDataViz, "1", 10);
     
also make sure to include `#include "sub/SubWrapperHelper.h"`

## How to use Pub(Publication)

If you application needs to send data then **Pub** is what you need. In order to use **Pub** you need to identify the following:
 - Skeleton type that delivers the data you need (_skeleton)
 - Skeleton identifier string (_id)
 - Event type within _skeleton that you need to publish to (_event)
 - Data type that will be published by your app (_dataType)
 - logger  (_logger)

After identifying above follow these 3 steps:

 1. You need to create a derived class that inherits from **PubDataWrapper** and Implement **GetData** method, **GetData** method is the method that gets called when ever publisher wants to send data. you need to prepare your raw data into final format in this method.
 2. You need to create a derived class that inherits from **PubWrapper**  and implement **GetEvent** Function, also make sure you provide **PubWrapper** constructor with proper data and **PubDataWrapper** derived class from 1.
 3. You need to call **Init** method on the class you just created in 2 in your application.
 
**Here is an example:**
in this example 

> _skeleton = VisualizationSkeleton 
> _id =  lidar_hesai_left2_server/lidar_hesai_left2_server/LidarHesaiLeft2PPort
> _event = LidarHesaiLeft2PPC2Viz
> _dataType = HesaiPointCloud
> _logger = logger

Step 1:

	#include "pub/PubDataWrapper.h"
    template <typename EventType>
    class SamplePubDataWrapper : public PubDataWrapper<EventType>
    {
    public:
    	void GetData(ara::com::SampleAllocateePtr<EventType> data) override
    	{
    		data->header.frameId = 0xA5;
    		std::cout << "[Wrapper test]############# " <<  "Getting data ready to publish" << std::endl;
    	}
    };
---
Step 2:

    #include "pub/PubWrapper.h"
    
    using Skeleton = huawei::ap::sample::skeleton::VisualizationSkeleton; 
    using Event = ara::com::internal::skeleton::event::EventAdapter<::HesaiPointCloud>;	
    class PubSample : public PubWrapper<Skeleton, Event, HesaiPointCloud>	
    {	
    public:	
    	SamplePubDataWrapper<HesaiPointCloud> s;	
    	PubSample(const std::string& name, ara::log::Logger& log) :	SkeletonWrapper(s, name, log) {}	
    private:	
    	Event& GetEvent() override	
    	{	return _skeleton->LidarHesaiLeft2PPC2Viz;	} 
    };
   ---
   Step 3:

    int main()
    {  
    	PubSample s("lidar_hesai_left2_server/lidar_hesai_left2_server/LidarHesaiLeft2PPort", _logger);
    	s.Init();
    
    	while(1)
    	{
    		std::this_thread::sleep_for(std::chrono::milliseconds(10));
    	}
    }

Then You can call **Publish** on s object whenever you like to send data

## Pub Helper

Instead of doing step 2 there is a macro in code **GENERATE_PUB_WRAPPER** that can generate the  code for you given the proper parameters.

It is recommended that you use this macro for speed of development

Step 2:

     GENERATE_PUB_WRAPPER(PubSample, SamplePubDataWrapper, VisualizationSkeleton, HesaiPointCloud, LidarHesaiLeft2PPC2Viz)

also make sure to include `"skeleton/SkeletonWrapperHelper.h"`




